//*******************************************************
//
//	IrCtrl2
//
//	シリアル通信
//
//	MCU : ATmega88
//		efuse:F5h
//		hfuse:D5h	BOD 2.7V
//		lfuse:E7h	4MHz セラロック
//
//	Compiler:	AVR-GCC		4.8.2
//				AVR-LIBC	1.8.0
//				binutils	2.23.1
//
//	2014.6.14	KAWAKAMI Yukio
//*******************************************************

#include	<string.h>
#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>

#include	"main.h"
#include	"com.h"

#ifndef	UDR0
#define	UDR0	UDR
#define	UCSR0A	UCSRA
#define	UCSR0B	UCSRB
#define	UCSR0C	UCSRC
#define	UBRR0	UBRR
#define	UBRR0H	UBRRH
#define	UBRR0L	UBRRL
#define	RXC0	RXC
#define	TXC0	TXC
#define	UDRE0	UDRE
#define	FE0		FE
#define	DOR0	DOR
#define	UPE0	UPE
#define	U2X0	U2X
#define	RXCIE0	RXCIE
#define	TXCIE0	TXCIE
#define	UDRIE0	UDRIE
#define	RXEN0	RXEN
#define	TXEN0	TXEN
#define	UCSZ02	UCSZ2
#define	RXB80	RXB8
#define	TXB80	TXB8
#define	UCSZ01	UCSZ1
#define	UCSZ00	UCSZ0
#endif

extern COM_WORK Com0;

PROGMEM const ushort BaudReg[12] = {
	(ushort)(F_CPU / 16 / 300L - 1),
	(ushort)(F_CPU / 16 / 600L - 1),
	(ushort)(F_CPU / 16 / 1200L - 1),
	(ushort)(F_CPU / 16 / 2400L - 1),
	(ushort)(F_CPU / 16 / 4800L - 1),
	(ushort)(F_CPU / 16 / 9600L - 1),
	(ushort)(F_CPU / 16 / 14400L - 1),
	(ushort)(F_CPU / 16 / 19200L - 1),
	(ushort)(F_CPU / 16 / 28800L - 1),
	(ushort)(F_CPU / 16 / 38400L - 1),
	(ushort)(F_CPU / 16 / 57600L - 1),
	(ushort)(F_CPU / 16 / 115200L - 1)
};

#if RX_BUFF_SIZE > 0
//********************************************************************
//	シリアル 受信割り込み (Com0)
//********************************************************************
ISR(USART_RX_vect){

	if (UCSR0A & ((1<<FE0)|(1<<DOR0)|(1<<UPE0))){	// 受信エラー
		volatile uchar	dummy = dummy;
		dummy = UDR0;	// 読み捨て
	} else {
		Com0.ibuff[Com0.ibuff_in++] = UDR0;
		if (Com0.ibuff_in >= sizeof(Com0.ibuff)){
			Com0.ibuff_in = 0;
		}
		(Com0.ibuff_remain)++;
	}
}
#endif

#if TX_BUFF_SIZE > 0
//********************************************************************
//	シリアル送信データレジスタ空き割り込み (Com0)
//********************************************************************
ISR(USART_UDRE_vect){

	if (Com0.obuff_remain){
		UDR0 = Com0.obuff[Com0.obuff_out++];
		if (Com0.obuff_out >= sizeof(Com0.obuff)){
			Com0.obuff_out = 0;
		}
		--(Com0.obuff_remain);
	}

	if (!Com0.obuff_remain){
		UCSR0B &= ~(1<<UDRIE0);	// 送信データレジスタ空き割り込み禁止
	}
}
#endif


//********************************************************************
//	シリアルポート初期化
//********************************************************************
void init_com(COM_WORK *com, COM_BAUD baud){

	cli();	// 割り込み禁止

	if (com == &Com0){
		ushort	b;

		UCSR0A = 0;

		b = pgm_read_word(&BaudReg[baud]);
#if F_CPU == 4000000
		if (baud == B38400){
			UBRR0H = 0;
			UBRR0L = 12;
			UCSR0A |= (1<<U2X0);
		} else {
			UBRR0H = (uchar)(b >> 8);
			UBRR0L = (uchar)b;
		}
#else
		UBRR0H = (uchar)(b >> 8);
		UBRR0L = (uchar)b;
#endif

#if RX_BUFF_SIZE > 0
		UCSR0B = (1<<RXCIE0)|(1<<RXEN0)|(1<<TXEN0);	// 送受信許可、受信完了割り込み許可
#else
		UCSR0B = (1<<RXEN0)|(1<<TXEN0);		// 送受信許可
#endif
		UCSR0C = (1<<UCSZ01)|(1<<UCSZ00);	// Stop1 8bit non
	}

	memset((void *)com, 0, sizeof(COM_WORK));

	sei();	// 割り込み許可
}


//********************************************************************
//	受信バッファに入っているデータの byte数取得
//********************************************************************
uchar check_com(COM_WORK *com){

#if RX_BUFF_SIZE > 0
	return com->ibuff_remain;
#else
	if (UCSRA & (1<<RXC)){
		return 1;
	} else {
		return 0;
	}
#endif
}


//********************************************************************
//	シリアルポート受信バッファから 1byte読み込み
//********************************************************************
unsigned char read_com(COM_WORK *com){

#if RX_BUFF_SIZE > 0
	unsigned char	data;

	// 受信データが無ければ来るまで待つ
	while(!com->ibuff_remain){
		WDR;
		SLEEP;
	}

	// 受信バッファからデータ読み出し
	data = com->ibuff[com->ibuff_out++];
	if (com->ibuff_out >= sizeof(com->ibuff)){
		com->ibuff_out = 0;
	}
	cli();
	--com->ibuff_remain;
	sei();
	return data;

#else
	while(!(UCSRA & (1<<RXC))){
		WDR;
	}
	return UDR;
#endif

}


//********************************************************************
//	シリアルポート 1byte送信
//********************************************************************
void write_com(COM_WORK *com, unsigned char data){

#if TX_BUFF_SIZE > 0
	// 送信バッファがいっぱいなら待つ
	while(com->obuff_remain >= sizeof(com->obuff)){
		WDR;
		SLEEP;
	}

	// データを送信バッファに積む
	com->obuff[com->obuff_in++] = data;
	if (com->obuff_in >= sizeof(com->obuff)){
		com->obuff_in = 0;
	}
	cli();
	com->obuff_remain++;
	sei();

	if (com == &Com0){
		// 送信データレジスタ空き割り込み許可
		UCSR0B |= (1<<UDRIE0);
	}

#else
	if (com == &Com0){
		// 送信データレジスタ空きフラグチェック
		while(!(UCSR0A & (1<<UDRE0)));
		UDR0 = data;
	}
#endif
}


//********************************************************************
//	RAM上文字列送信
//********************************************************************
void write_str(COM_WORK *com, uchar *str){
	unsigned char	d;

	while((d = *str++)){
		write_com(com, d);
	}
}


//********************************************************************
//	固定文字列送信
//********************************************************************
void write_str_P(COM_WORK *com, PGM_P str){
	unsigned char	d;

	while((d = pgm_read_byte(str++))){
		write_com(com, d);
	}
}


