#ifndef __COM_H__
#define	__COM_H__
//*******************************************************
//
//	IrCtrl2
//
//	シリアル通信
//
//	MCU : ATmega88
//		efuse:F5h
//		hfuse:D5h	BOD 2.7V
//		lfuse:E7h	4MHz セラロック
//
//	Compiler:	AVR-GCC		4.8.2
//				AVR-LIBC	1.8.0
//				binutils	2.23.1
//
//	2014.6.14	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>

#define	RX_BUFF_SIZE	32
#define	TX_BUFF_SIZE	64

typedef struct S_COM_WORK {
#if RX_BUFF_SIZE > 0
	volatile uchar	ibuff[RX_BUFF_SIZE];
	volatile uchar	ibuff_in;
	volatile uchar	ibuff_out;
	volatile uchar	ibuff_remain;
#endif

#if TX_BUFF_SIZE > 0
	volatile uchar	obuff[TX_BUFF_SIZE];
	volatile uchar	obuff_in;
	volatile uchar	obuff_out;
	volatile uchar	obuff_remain;
#endif
} COM_WORK;

typedef enum E_COM_BAUD {
	B300 = 0,
	B600,
	B1200,
	B2400,
	B4800,
	B9600,
	B14400,
	B19200,
	B28800,
	B38400,
	B57600,
	B115200
} COM_BAUD;


void	init_com(COM_WORK *com, COM_BAUD baud);
uchar	check_com(COM_WORK *com);
uchar	read_com(COM_WORK *com);
void	write_com(COM_WORK *com, uchar data);
void	write_str(COM_WORK *com, uchar *str);
void	write_str_P(COM_WORK *com, PGM_P str);

#endif	// __COM_H__
