//*******************************************************
//
//	IrCtrl2
//
//	MCU : ATmega88
//		efuse:F5h
//		hfuse:D5h	BOD 2.7V
//		lfuse:E7h	4MHz セラロック
//
//	Compiler:	AVR-GCC		4.8.2
//				AVR-LIBC	1.8.0
//				binutils	2.23.1
//
//	2014.9.9	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"main.h"
#include	"com.h"
#include	"ir.h"
#include	"task.h"


COM_WORK	Com0;

volatile uchar	WTimer;
volatile uchar	PowerOffTimer;		// 電源ボタンのON時間
volatile uchar	RecOffTimer;		// 録画ボタンのON時間
volatile uchar	PowerLedTimer;		// 電源LEDの点灯判断
volatile uchar	PowerLedTimerDS;
volatile uchar	PowerLedOnTimer;	// 電源LEDの点滅判断
volatile uchar	RecLedTimer;		// 録画LEDの点灯判断
volatile uchar	RecLedTimerDS;
volatile uchar	RecTimer;			// 録画タイミング
volatile uchar	RecTimerDS;
volatile uchar	IrTimer;

volatile uchar StatTimer;			// シーケンス処理タイマー
volatile uchar StatTimerDS;

extern uchar	Command;

extern const PROGMEM IR_DATA TunerCodePowerOn;
extern const PROGMEM IR_DATA RecoderCodePowerOn;
extern const PROGMEM IR_DATA RecoderCodeReturn;
extern const PROGMEM IR_DATA RecoderCodeOK;

//*******************************************************
//	タイマー0比較一致A割り込み
//	100Hz
//*******************************************************
ISR(TIMER0_COMPA_vect){

	sei();	// 他からの割り込み許可

	if (WTimer)		--WTimer;
	if (IrTimer)	--IrTimer;

	if (PowerOffTimer){		// 電源ボタンの押下時間管理
		if (--PowerOffTimer == 0){
			POWER_SW_OFF;
		}
	}

	if (RecOffTimer){		// 録音ボタンの押下時間管理
		if (--RecOffTimer == 0){
			REC_SW_OFF;
		}
	}

	if (StatTimer){			// シーケンスの時間管理
		if (++StatTimerDS >= 100){
			StatTimerDS = 0;
			--StatTimer;
		}
	}

	if (RecTimer){			// 録画開始時間管理
		if (++RecTimerDS >= 100){
			RecTimerDS = 0;
			--RecTimer;
		}
	}

	if (IS_POWER_LED){		// POWER LED点灯検出
		PowerLedTimer = 3;
		PowerLedTimerDS = 0;
		if (PowerLedOnTimer < 0xFF)	PowerLedOnTimer++;
	} else {
		PowerLedOnTimer = 0;
		if (PowerLedTimer){
			if (++PowerLedTimerDS >= 100){
				PowerLedTimerDS = 0;
				--PowerLedTimer;
			}
		}
	}

	if (IS_REC_LED){			// 録画LED点灯検出
		RecLedTimer = 3;
		RecLedTimerDS = 0;
	} else if (RecLedTimer){
		if (++RecLedTimerDS >= 100){
			RecLedTimerDS = 0;
			--RecLedTimer;
		}
	}
}


//*******************************************************
//	メインルーチン
//*******************************************************
int main(void){

	cli();
	
	PORTB = 0xFE;	// ~(1<<LED)
	DDRB = (1<<LED);

	PORTC = (1<<LIN1)|(1<<LIN2)|(1<<4)|(1<<5);
	DDRC = (1<<SOUT1)|(1<<SOUT2);

	PORTD = (1<<5)|(1<<4)|(1<<2)|(1<<TXD)|(1<<RXD);
	DDRD = (1<<DEBUG_PORT)|(1<<IR_OUT);

	MCUCR = (1<<SE);		// スリープ許可、アイドリングモード

	// TIMER0
	TCCR0A = (1<<WGM01);				// CTC
	TCCR0B = (1<<CS02);					// プリスケーラ 1/256
	OCR0A = ((F_CPU/256 + 50)/100 - 1);	// 100Hz
	TIMSK0 = (1<<OCIE0A);				// 比較一致A割り込み許可

	// TIMER1
	TCCR1A = 0;						// 標準動作
	TCCR1B = (1<<CS11)|(1<<CS10);	// プリスケーラ 1/64
	TIMSK1 = (1<<TOIE1);			// オーバーフロー割り込み許可

	// TIMER2
	TCCR2A = (1<<WGM21)|(1<<WGM20);				// 高速PWM
	TCCR2B = (1<<WGM22)|(1<<CS20);				// プリスケーラなし
	OCR2A = ((F_CPU + (38000L/2))/38000L) - 1;	// 38kHz
	OCR2B = (((F_CPU + (38000L/2))/38000L) - 1) / 3;
	TIMSK2 = (1<<OCIE2B);						// 比較一致B割り込み許可

	// アナログコンパレータ
	ACSR = (1<<ACBG)|(1<<ACIE);	// 内部基準電圧、アナログ比較割り込み許可、両エッジ
	DIDR1 = (1<<AIN1D);		// AIN1デジタル入力禁止

	// 電力削減
	PRR = (1<<PRTWI)|(1<<PRSPI)|(1<<PRADC);
	
	sei();

	LED_ON;

	wait_timer(30);

	init_com(&Com0, B38400);
	cputs(PSTR("IrCtrl " VERSION "\r\n"));

	LED_OFF;

	PowerLedTimer = 0;
	PowerLedOnTimer = 0;
	RecLedTimer = 0;

	for(;;){

		WDR;

		job_ir();
		job_task();

		// 外部制御
		if (check_com(&Com0)){
			uchar	c;
			c = read_com(&Com0);
			if (c == 'o'){
				Command = COMMAND_START;
			} else if (c == 'f'){
				Command = COMMAND_POWEROFF;
			} else if (c == 'r'){
				Command = COMMAND_REC;
			} else if (c == 's'){
				Command = COMMAND_STOP;
			} else if (c == 'p'){
				ir_send(&RecoderCodePowerOn);
			} else if (c == 't'){
				ir_send(&TunerCodePowerOn);
			}
		}

	}
}


//*******************************************************
//	固定メッセージ表示
//*******************************************************
void cputs(PGM_P msg){

	write_str_P(&Com0, msg);
}


//*******************************************************
//	10ms単位時間待ち
//*******************************************************
void wait_timer(uchar tm){

	WTimer = tm;

	while(WTimer){
		WDR;
		SLEEP;
	}
}


